@@ -147,7 +147,8 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u't',u'(.*)tA',u'pata'),
 			(u'k>j',u'(.*A)ikA',u'aika'),
 			(u'k>',u'(.*AA)kA',u'raaka'),
-			(u'k>',u'(.*)kA',u'haka')]),
+			(u'k>',u'(.*V)kA',u'liika'),
+			(u'k>',u'(.*C)kA',u'haka')]),
 	(u'nahka', u'-', [(None,u'(.*)kA',u'nahka')]),
 	(u'jumala', u'-', [(None,u'(.*l)A',u'jumala')]),
 	(u'koira', u'sw',   [(None,u'(.*)A',u'koira'),
@@ -163,6 +164,7 @@ modern_classmap = [(u'valo', u'sw', [(None,u'(.*)',u'valo'),
 			(u't',u'(.*)tA',u'juhta'),
 			(u'k>',u'(.*i)kA',u'ikä'),
 			(u'k>',u'(.*)kA',u'hoikka')]),
+	(u'ylkä', u'-', [(None,u'(.*l)kA',u'ylkä')]),
 	(u'pitkä', u'-', [(None,u'(.*pi)tkA',u'pitkä')]),
 	(u'ruoka', u'-', [(None,u'(.*ru)oka',u'ruoka')]),
 	(u'poika', u'-', [(None,u'(.*po)ikA',u'poika')]),
