@@ -45,6 +45,7 @@
 # date; cat ~/Lataukset/koesanat?.txt | flookup -i sukija.fst | gawk 'length($0) > 0' | sort >test.out; date
 # diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '<.*[+][?]' | less
 # diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '>.*[+][?]' | less
+# diff test.out ~/Lataukset/vvfst-sukija-testi.out | grep '>.*[+][?]' | gawk '{print $2}' |flookup -i sukija.fst | gawk 'length($0) > 0'
 # cp test.out ~/Lataukset/vvfst-sukija-testi.out
 
 # Style- ja usage-lippujen arvot suoraan Joukahaisesta:
@@ -104,6 +105,8 @@ re_omi = makeRe (u"Ln", u".Comi")
 re_oni = makeRe (u"Ln", u".Coni")
 re_ori = makeRe (u"Ln", u".Cori")
 
+re_OOri = makeRe (u"Ln", u".Cööri")
+
 re_adinen = makeRe (u"Ln", u".Cadinen")
 re_odinen = makeRe (u"Ln", u".Codinen")
 re_ofinen = makeRe (u"Ln", u".Cofinen")
@@ -128,7 +131,7 @@ re_oitin = makeRe (u"Ln", u".Coitin")
 re_aatio = makeRe (u"Ln", u".Caatio")
 re_uutio = makeRe (u"Ln", u".Cuutio")
 re_uusio = makeRe (u"Ln", u".Cuusio")
-re_tio   = makeRe (u"Ln", u"[ik]tio") # Traditio, funktio.
+re_tio   = makeRe (u"Ln", u"([^a]i|k)tio") # Traditio, funktio, mutta ei aitio.
 
 re_toninen = makeRe (u"Ll", u".toninen")
 re_iivinen = makeRe (u"Ll", u"Ciivinen")
@@ -143,15 +146,13 @@ re_eikAs = makeRe (u"Ll", u"eikAs")
 
 # Words to be excluded.
 #
-re_adi_x = re.compile (u"\\A\[Ln\]\[Xp\](dekadi|faradi|pikofaradi|stadi)\[X\]")
+re_adi_x = re.compile (u"\\A\[Ln\]\[Xp\](faradi|pikofaradi|stadi)\[X\]")
 re_ogi_x = re.compile (u"\\A\[Ln\]\[Xp\](blogi|grogi|judogi)\[X\]")
 re_omi_x = re.compile (u"\\A\[Ln\]\[Xp\](binomi|bromi|dibromi|genomi|kromi|trinomi)\[X\]")
 re_oni_x = re.compile (u"\\A\[Ln\]\[Xp\](ikoni)\[X\]")
 re_ori_x = re.compile (u"\\A\[Ln\]\[Xp\](hevosori|jalostusori|reettori|siitosori)\[X\]")
 
 re_logia_x = re.compile (u"\\A\[Ln\]\[Xp\](genealogia|trilogia)\[X\]")
-re_uusio_x = re.compile (u"\\A\[Ln\]\[Xp\](diffuusio)\[X\]")
-re_tio_x   = re.compile (u"\\A\[Ln\]\[Xp\](aitio)\[X\]")
 
 re_A = re.compile (u"[aou]")
 
@@ -166,6 +167,8 @@ spelling_pattern_list = [
   (re_oni, u"on", u"oon", re_oni_x),  # Telefoni  => telefooni.
   (re_ori, u"or", u"oor", re_ori_x),  # Pehtori   => pehtoori.
 
+  (re_OOri,     u"öör",   u"ör"),     # Amatööri => amatöri.
+
   (re_adinen,   u"adi",    u"aadi"),
   (re_odinen,   u"odi",    u"oodi"),
   (re_ofinen,   u"ofi",    u"oofi"),
@@ -181,10 +184,6 @@ spelling_pattern_list = [
   (re_loginen,  u"logi",   u"loogi"),
 
   (re_oitin, u"oit", u"ot"),  # Kirjoitin => kirjotin (esim. kirjo(i)ttimen).
-  (re_aatio, u"aatio", u"atsio", u"atsion", u"atsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
-  (re_uutio, u"uutio", u"utsio", u"utsion", u"utsioon", u"NimisanaAutio_a", u"NimisanaPaperi_a"),
-  (re_uusio, u"uusio", u"usio",  u"usion",  u"usioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_uusio_x),
-  (re_tio,   u"tio",   u"tsio",  u"tsion",  u"tsioon",  u"NimisanaAutio_a", u"NimisanaPaperi_a", re_tio_x),
   (re_oittaa1, u"o",   u"ot",  u"Kirjoittaa", u"Alittaa"),
   (re_oittaa2, u"ö",   u"öt",  u"Kirjoittaa", u"Alittaa"),
   (re_oittaa1, u"oit", u"ot",  u"Alittaa",    u"Alittaa"),
@@ -218,24 +217,20 @@ def word_class (line):
 # Tuomo Tuomi: Suomen kielen käänteissanakirja, 2. painos.
 # Suomalaisen Kirjallisuuden Seura 1980.
 
+def write_word (line, word, lexicon):
+    prefix = line[0:line.find (u" ")]
+    A = u"a" if re_A.search(word) else u"ä"
+    outfile.write (u"%s %s%s_%s ;\n" % (prefix, word_class(line), lexicon, A))
 
 def write_ahven (line, word):
-    prefix = line[0:line.find (u"[X]")+3]
-    A = u"a" if re_A.search(word) else u"ä"
-    outfile.write (u"%s%s[Ses][Ny]n%s:%sn%s NimisanaLiOlV_%s ;\n" % (prefix, word, A, word, A, A))
-
+    if not line.startswith (u"[Lu]"):
+        write_word (line, word, u"SukijaAhven")
 
 def write_kaunis (line, word):
-    prefix = line[0:line.find (u" ")]
-    A = u"a" if re_A.search(word) else u"ä"
-    outfile.write (u"%s %sSukijaKaunis_%s ;\n" % (prefix, word_class(line), A))
-
+    write_word (line, word, u"SukijaKaunis")
 
 def write_altis (line, word):
-    prefix = line[0:line.find (u" ")]
-    A = u"a" if re_A.search(word) else u"ä"
-    outfile.write (u"%s %sSukijaAltis_%s ;\n" % (prefix, word_class(line), A))
-
+    write_word (line, word, u"SukijaAltis")
 
 def write_virkkaa (line, word):
     prefix = line[0:line.find (u" ")]
@@ -249,8 +244,11 @@ def write_paahtaa (line, word):
     prefix = line[0:line.find (u" ")]
     outfile.write (u"%s SukijaPaahtaa_a ;\n" % (prefix))
 
+def write_lahti (line, word):
+    write_word (line, word, u"SukijaLahti")
 
-def generate_from_pattern (line, pattern_list):
+
+def generate_from_pattern_1 (line, pattern_list):
     for x in pattern_list:
         if x[0].match(line):
             if (len(x) == 2):
@@ -258,12 +256,15 @@ def generate_from_pattern (line, pattern_list):
             elif (len(x) == 3) or (len(x) == 4 and not x[3].match(line)):
                 replace_and_write (line, x[1], x[2])
             elif (len(x) == 5) and (line.find (x[3]) >= 0):
-                replace_and_write (line.replace(x[3], x[4]), x[1], x[2])
-            elif (len(x) == 7) or (len(x) == 8 and not x[7].match(line)):
-                s = line.replace (x[5], x[6])
-                outfile.write (replace (s, x[1], x[3]))
-                outfile.write (replace (s, x[1], x[4]))
-                outfile.write (replace (line, x[1], x[2]))
+                replace_and_write (line.replace(x[3],x[4]), x[1], x[2])
+
+
+def generate_from_pattern_2 (line, pattern, string, p1, p2, s1, s2):
+    if pattern.match (line):
+        for x in p1:
+            replace_and_write (line, string, x)
+        for x in p2:
+            replace_and_write (line.replace(s1,s2), string, x)
 
 
 # Old spellings and common spelling errors of words
@@ -278,6 +279,7 @@ word_list = [
     (u"ainainen",         (u"ainai",         u"ainahi")),
     (u"aineisto",         (u"aineisto",      u"ainehisto")),
     (u"aksiomi",          (u"aksiom",        u"aksioom")),
+    (u"aksiooma",         (u"aksioom",       u"aksiom")),
     (u"alamainen",        (u"alamai",        u"alammai")),
     (u"alimmainen",       (u"alimmai",       u"alimai")),
     (u"alkovi",           (u"alkov",         u"alkoov")),
@@ -289,10 +291,13 @@ word_list = [
     (u"Australia",        (u"Australi",      u"Austraali")),
     (u"barbaari",         (u"barbaar",       u"barbar")),
     (u"beduiini",         (u"beduiin",       u"beduin")),
-    (u"borssi",           (u"borssi",        u"borsh")),
-    (u"dervissi",         (u"derviss",       u"dervish")),
+    (u"borssi",           (u"borss",         u"borsh")),
+    (u"dervissi",         (u"derviss",       u"dervish", u"dervisch")),
     (u"divaani",          (u"divaan",        u"divan")),
+    (u"drakma",           (u"drakm",         u"drakhm")),
+    (u"edes",             (u"edes",          u"ees")),
     (u"emali",            (u"emal",          u"emalj")),
+    (u"emaloida",         (u"emalo",         u"emaljo")),
     (u"ensimmäinen",      (u"ensimmäi",      u"ensimäi")),
     (u"eteinen",          (u"etei",          u"etehi")),
     (u"evankelinen",      (u"evankeli",      u"evankeeli")),
@@ -307,6 +312,7 @@ word_list = [
     (u"inkvisitio",       (u"inkvisitio",    u"inkvisiitio")),
     (u"insinööri",        (u"insinöör",      u"insinör")),
     (u"invalidi",         (u"invalid",       u"invaliid")),
+    (u"janitsaari",       (u"janitsaar",     u"janitsar", u"janitschar", u"janitschaar", u"janitshaar")),
     (u"juridinen",        (u"juridi",        u"juriidi")),
     (u"kahdeksainen",        (u"kahdeksai",         u"kaheksai")),
     (u"kahdeksan",           (u"kahdeks",           u"kaheks")),
@@ -325,6 +331,7 @@ word_list = [
     (u"kanuuna",        (u"kanuun",      u"kanun")),
     (u"kaoottinen",     (u"kaootti",     u"kaaootti", u"kaaotti", u"kaotti")),
     (u"Kaukasia",       (u"Kaukasi",     u"Kaukaasi")),
+    (u"kaunoinen",      (u"kaunoi",      u"kaunohi")),
     (u"katolilainen",   (u"katolilai",   u"katoolilai")),
     (u"katolinen",      (u"katoli",      u"katooli")),
     (u"kavaljeeri",     (u"kavaljeer",   u"kavalier", u"kavaljier")),
@@ -334,6 +341,7 @@ word_list = [
     (u"kollegio",       (u"kollegio",    u"kolleegio")),
     (u"koneisto",       (u"koneisto",    u"konehisto")),
     (u"konossementti",  (u"konossement", u"konnossement")),
+    (u"konttori",       (u"konttor",     u"kontoor")),
     (u"kortteeri",      (u"kortteer",    u"kortter", u"kortier", u"korttier")),
     (u"kraatteri",      (u"kraatter",    u"kraater", u"krateer")),
     (u"kritiikki",      ((u"kritiik",    u"kritik", u"NimisanaTakki_ä", u"NimisanaRisti_ä"),
@@ -342,6 +350,7 @@ word_list = [
     (u"kuriiri",        (u"kuriir",      u"kurier")),
     (u"kurtiini",       (u"kurtiin",     u"kurtin")),
     (u"kuvernööri",     (u"kuvernöör",   u"kuvernör")),
+    (u"laboratorio",    (u"laboratorio", u"laboratoorio")),
     (u"leegio",         (u"leegio",      u"legio")),
     (u"legioona",       (u"legioon",     u"legion")),
     (u"legioonalainen", (u"legioonalai", u"legionalai")),
@@ -403,6 +412,7 @@ word_list = [
     (u"serafi",          (u"seraf",       u"seraaf")),
     (u"shampanja",       (u"shampanj",    u"shamppanj")),
     (u"sihteeri",        (u"sihteer",     u"sihter", u"sihtier")),
+    (u"sikari",          (u"sikar",       u"sikaar")),
     (u"sitruuna",        (u"sitruun",     u"sitrun", u"sitroon", u"sitron")),
     (u"siviili",         (u"siviil",      u"sivil")),
     (u"slaavilainen",    (u"slaavilai",   u"slavilai")),
@@ -436,8 +446,8 @@ word_list = [
     (u"yhdeksäs",           (u"yhdeksä",          u"yheksä")),
     (u"ylimmäinen",         (u"ylimmäi",          u"ylimäi")),
 
-    (u"ien",     [u"[Ln][Xp]ien[X]iken[Sn][Ny]e:ikene Loppu ;"]),
-##                  u"[Ln][Xp]ien[X]ien[Ses][Ny]nä:iennä NimisanaLiOlV_ä ;"
+    (u"ien",     [u"[Ln][Xp]ien[X]iken[Sn][Ny]e:ikene Loppu ;",
+                  u"[Ln][Xp]ien[X]ien[Ses][Ny]nä:iennä NimisanaLiOlV_ä ;"]),
 
     (u"kappale", [u"[Ln][Xp]kappale[X]kappal[Sg][Nm]ten:kappalten # ;"]),
     (u"maailma", [u"[Ln][Xp]maailma[X]maailmoitse:maailmoitse # ;"]),
@@ -473,6 +483,9 @@ word_list = [
     (u"vetre", [u"[Ll][Xp]vetre[X]vetre[Ses][Ny]nnä:vetrennä NimisanaLiOlV_ä ;"]),
     (u"päre",  [u"[Ln][Xp]päre[X]päre[Ses][Ny]nnä:pärennä NimisanaLiOlV_ä ;"]),
     (u"terve", [u"[Lnl][Xp]terve[X]terve[Ses][Ny]nnä:tervennä NimisanaLiOlV_ä ;"]),
+
+    (u"kaivu", [u"[Ln][Xp]kaivu[X]kaivu:kaivu NimisanaPuu_a ;",
+                u"[Ln][Xp]kaivu[X]kaivu[Sill][Ny]usee:kaivuusee NimisanaLiOlN_a ;"]),
 ]
 
 
@@ -494,6 +507,38 @@ function_list = [
       u"saarua",
       u"tanhua")),
 
+# Vapaa ja tienoo ovat taivutuskaavoina SukijaVapaa.
+#
+    # 23 vapaa (8, 8). Tuomi, s. 1, 2.
+    #
+#    (write_vapaa_tienoo,
+#     (u"kajaa",
+#      u"vajaa",
+#      u"vakaa",
+#      u"suklaa",
+#      u"harmaa",
+#      u"vapaa",
+#      u"nepaa",
+#      u"hurraa")),
+
+    # 24 tienoo (14, 14). Tuomi, s. 345. Taipuu kuten vapaa.
+    #
+#    (write_vapaa_tienoo,
+#     (u"kabeljoo",
+#      u"kalikoo",
+#      u"pikoo",
+#      u"talkoo",
+#      u"haloo",
+#      u"halloo",
+#      u"tienoo",
+#      u"poppoo",
+#      u"bigarroo",
+#      u"platoo",
+#      u"ehtoo",
+#      u"palttoo",
+#      u"ponttoo",
+#      u"nivoo")),
+
     # 33 lohi (2, 2). Tuomi, s. 151.
     # lohten, uuhten
     #
@@ -503,6 +548,13 @@ function_list = [
       u"tyynenmerenlohi",   # On Joukahaisessa.
       u"uuhi")),
 
+    # 34 lahti (2, 2). Tuomi, s. 193.
+    # lahta (= lahtea), lahtein
+    #
+    (write_lahti, 
+     (u"haahti",
+      u"lahti")),
+
     # Ahven taipuu kuten sisar, paitsi että yksikön olento on myös ahvenna.
     #
     # 55 ahven (22, 23). Tuomi, s. 246, 247, 301, 302.
@@ -514,7 +566,7 @@ function_list = [
       u"huomen",
       u"häiven",
       u"höyhen",
-      u"ien",
+#      u"ien",  # On erikseen: ikene, ien.
       u"iljen",
       u"joutsen",
       u"jäsen",
@@ -637,21 +689,34 @@ def generate_xiljoona (line):
             outfile.write (line.replace (u"iljoonien", u"iljonain"))
 
 
+ei_vertm = re.compile (u"@[PDC][.]EI_VERTM([.]ON)?@", re.UNICODE)
+
 # Copy Voikko vocabulary and insert forms that Sukija needs.
 #
 while True:
     line = infile.readline()
     if line == u"":
         break
+    line = re.sub (ei_vertm, u"", line)
     if OPTIONS["sukija-ys"]:
-        line = line.replace (u"@P.YS_EI_JATKOA.ON@", u"")
+        line = line.replace (u"@P.YS_EI_JATKOA.ON@" , u"")
+        line = line.replace (u"@D.YS_EI_JATKOA@", u"")
+        line = line.replace (u"@C.YS_EI_JATKOA@", u"")
     outfile.write (line)
     if line == u"LEXICON Sanasto\n":
         outfile.write (u"Sukija ;\n")
     if line == u"LEXICON Joukahainen_p\n":
         outfile.write (u"SukijaEtuliite ;\n")
+    if line == u"LEXICON LukusananErikoisjälkiliite\n":
+        outfile.write (u"SukijaLukusananErikoisjälkiliite ;\n")
+
+    generate_from_pattern_1 (line, spelling_pattern_list)
+
+    generate_from_pattern_2 (line, re_uusio, u"uusio", (u"usio",),          (u"usion",  u"usioon"),  u"NimisanaAutio_a", u"NimisanaPaperi_a")
+    generate_from_pattern_2 (line, re_tio,   u"tio",   (u"tsio",),          (u"tsion",  u"tsioon"),  u"NimisanaAutio_a", u"NimisanaPaperi_a")
+    generate_from_pattern_2 (line, re_aatio, u"aatio", (u"atio", u"atsio"), (u"atsion", u"atsioon"), u"NimisanaAutio_a", u"NimisanaPaperi_a")
+    generate_from_pattern_2 (line, re_uutio, u"uutio", (u"utio", u"utsio"), (u"utsion", u"utsioon"), u"NimisanaAutio_a", u"NimisanaPaperi_a")
 
-    generate_from_pattern (line, spelling_pattern_list)
 
     r = base_form_re.search (line)
     if r:
