@@ -56,7 +56,7 @@ def filterLines(lines, lexiconPrefix):
 			if lexiconPrefix == u"Nimisana":
 				yield line[13:].replace(u"<WC>", lexiconPrefix)
 		elif line.startswith("?NotLaatusana"):
-			if lexiconPrefix != u"Laatusana":
+			if lexiconPrefix not in [u"Laatusana", u"Asemosana"]:
 				yield line[13:].replace(u"<WC>", lexiconPrefix)
 		elif line.startswith("?Paikannimi"):
 			if lexiconPrefix == u"Paikannimi":
@@ -70,6 +70,12 @@ def filterLines(lines, lexiconPrefix):
 		elif line.startswith("?Nimi"):
 			if lexiconPrefix == u"Nimi":
 				yield line[5:].replace(u"<WC>", lexiconPrefix)
+		elif line.startswith("?Erisnimi"):
+			if lexiconPrefix in [u"Sukunimi", u"Etunimi", u"Paikannimi", u"Nimi"]:
+				yield line[9:].replace(u"<WC>", lexiconPrefix)
+		elif line.startswith("?Yleisnimi"):
+			if lexiconPrefix in [u"Nimisana", u"Laatusana", u"NimiLaatusana"]:
+				yield line[10:].replace(u"<WC>", lexiconPrefix)
 		elif line.startswith("?Asemosana"):
 			if lexiconPrefix == u"Asemosana":
 				yield line[10:].replace(u"<WC>", lexiconPrefix)
@@ -108,7 +114,8 @@ def appendLexicon(lexiconName, lines, lexcFile):
 		                u"NormaaliYsJatko", u"Nainen", u"NainenYhteiset", u"NainenYsJatko", u"Autio", \
 		                u"NormaaliYsJatkoOl", u"Luku", u"Valo_w", u"Valo_s", u"Valo_sl", \
 		                u"Koira", u"Koira_w", u"Koira_s", u"MonikonGenetiiviInJatko", \
-		                u"Koira_w_monikko", u"Koira_w_yksikkö", u"Puu", u"Maa_l", u"Maa_s"]:
+		                u"Koira_w_monikko", u"Koira_w_yksikkö", u"Puu", u"Maa_l", u"Maa_s", \
+		                u"OlVLoppu", u"MonikonNominatiivinJatko"]:
 			appendLines(u"LukusananJälkiliite", realName, lines, lexcFile)
 			appendLines(u"Asemosana", realName, lines, lexcFile)
 		if OPTIONS["sukija"] and realName in [u"SukijaYhteisetHMuodot"]:
@@ -189,6 +196,9 @@ for sija in MULTI.keys():
 	lexcFile.write(u"LEXICON " + lexiconName + u"Kertoimet\n")
 	for i in range(len(MULTI_VOWELS)):
 		lexcFile.write(diacritic + tagName + MULTI[sija][i] + u":" + diacritic + MULTI[sija][i] + u"\t" + lexiconName + MULTI_VOWELS[i] + u"\t;\n")
+		if MULTI[sija][i].endswith(u"joonien"):
+			altSija = MULTI[sija][i].replace(u"joonien", u"joonain")
+			lexcFile.write(diacritic + tagName + altSija + u":" + diacritic + altSija + u"\t" + lexiconName + MULTI_VOWELS[i] + u"\t;\n")
 	
 	numeralLines = []
 	numeralLines.append(diacritic + u":" + diacritic + u"\tLukusanaLiitesana_<A>\t;")
@@ -212,30 +222,30 @@ for sija in MULTI.keys():
 	appendLines(u"Lukusana", sija + u"1", numeralLines, lexcFile)
 
 MULTI_ORDINALS = {
-	u"SgNy": [u"kymmenennen", u"sadannen", u"tuhannennen", u"miljoonannen"],
-	u"SpNy": [u"kymmenettä", u"sadannetta", u"tuhannetta", u"miljoonannetta"],
-	u"StrNy": [u"kymmenenneksi", u"sadanneksi", u"tuhannenneksi", u"miljoonanneksi"],
-	u"SesNy": [u"kymmenentenä", u"sadantena", u"tuhannentena", u"miljoonantena"],
-	u"SineNy": [u"kymmenennessä", u"sadannessa", u"tuhannennessa", u"miljoonannessa"],
-	u"SelaNy": [u"kymmenennestä", u"sadannesta", u"tuhannennesta", u"miljoonannesta"],
-	u"SillNy": [u"kymmenenteen", u"sadanteen", u"tuhannenteen", u"miljoonanteen"],
-	u"SadeNy": [u"kymmenennellä", u"sadannella", u"tuhannennella", u"miljoonannella"],
-	u"SablNy": [u"kymmenenneltä", u"sadannelta", u"tuhannennelta", u"miljoonannelta"],
-	u"SallNy": [u"kymmenennelle", u"sadannelle", u"tuhannennelle", u"miljoonannelle"],
-	u"SabNy": [u"kymmenennettä", u"sadannetta", u"tuhannennetta", u"miljoonannetta"],
-	u"SgNm": [u"kymmenensien", u"sadansien", u"tuhannensien", u"miljoonansien"],
-	u"SpNm": [u"kymmenensiä", u"sadansia", u"tuhannensia", u"miljoonansia"],
-	u"StrNm": [u"kymmenensiksi", u"sadansiksi", u"tuhannensiksi", u"miljoonansiksi"],
-	u"SesNm": [u"kymmenensinä", u"sadansina", u"tuhannensina", u"miljoonansina"],
-	u"SineNm": [u"kymmenensissä", u"sadansissa", u"tuhannensissa", u"miljoonansissa"],
-	u"SelaNm": [u"kymmenensistä", u"sadansista", u"tuhannensista", u"miljoonansista"],
-	u"SillNm": [u"kymmenensiin", u"sadansiin", u"tuhannensiin", u"miljoonansiin"],
-	u"SadeNm": [u"kymmenensillä", u"sadansilla", u"tuhannensilla", u"miljoonansilla"],
-	u"SablNm": [u"kymmenensiltä", u"sadansilta", u"tuhannensilta", u"miljoonansilta"],
-	u"SallNm": [u"kymmenensille", u"sadansille", u"tuhannensille", u"miljoonansille"],
-	u"SabNm": [u"kymmenensittä", u"sadansitta", u"tuhannensitta", u"miljoonansitta"],
-	u"SinNm": [u"kymmenensin", u"sadansin", u"tuhannensin", u"miljoonansin"],
-	u"SkoNm": [u"kymmenensine", u"sadansine", u"tuhannensine", u"miljoonansine"]
+	u"SgNy": [u"kymmenennen", u"sadannen", u"tuhannennen", u"miljoonannen", u"miljardinnen"],
+	u"SpNy": [u"kymmenettä", u"sadannetta", u"tuhannetta", u"miljoonannetta", u"miljardinnetta"],
+	u"StrNy": [u"kymmenenneksi", u"sadanneksi", u"tuhannenneksi", u"miljoonanneksi", u"miljardinneksi"],
+	u"SesNy": [u"kymmenentenä", u"sadantena", u"tuhannentena", u"miljoonantena", u"miljardintena"],
+	u"SineNy": [u"kymmenennessä", u"sadannessa", u"tuhannennessa", u"miljoonannessa", u"miljardinnessa"],
+	u"SelaNy": [u"kymmenennestä", u"sadannesta", u"tuhannennesta", u"miljoonannesta", u"miljardinnesta"],
+	u"SillNy": [u"kymmenenteen", u"sadanteen", u"tuhannenteen", u"miljoonanteen", u"miljardinteen"],
+	u"SadeNy": [u"kymmenennellä", u"sadannella", u"tuhannennella", u"miljoonannella", u"miljardinnella"],
+	u"SablNy": [u"kymmenenneltä", u"sadannelta", u"tuhannennelta", u"miljoonannelta", u"miljardinnelta"],
+	u"SallNy": [u"kymmenennelle", u"sadannelle", u"tuhannennelle", u"miljoonannelle", u"miljardinnelle"],
+	u"SabNy": [u"kymmenennettä", u"sadannetta", u"tuhannennetta", u"miljoonannetta", u"miljardinnetta"],
+	u"SgNm": [u"kymmenensien", u"sadansien", u"tuhannensien", u"miljoonansien", u"miljardinsien"],
+	u"SpNm": [u"kymmenensiä", u"sadansia", u"tuhannensia", u"miljoonansia", u"miljardinsia"],
+	u"StrNm": [u"kymmenensiksi", u"sadansiksi", u"tuhannensiksi", u"miljoonansiksi", u"miljardinsiksi"],
+	u"SesNm": [u"kymmenensinä", u"sadansina", u"tuhannensina", u"miljoonansina", u"miljardinsina"],
+	u"SineNm": [u"kymmenensissä", u"sadansissa", u"tuhannensissa", u"miljoonansissa", u"miljardinsissa"],
+	u"SelaNm": [u"kymmenensistä", u"sadansista", u"tuhannensista", u"miljoonansista", u"miljardinsista"],
+	u"SillNm": [u"kymmenensiin", u"sadansiin", u"tuhannensiin", u"miljoonansiin", u"miljardinsiin"],
+	u"SadeNm": [u"kymmenensillä", u"sadansilla", u"tuhannensilla", u"miljoonansilla", u"miljardinsilla"],
+	u"SablNm": [u"kymmenensiltä", u"sadansilta", u"tuhannensilta", u"miljoonansilta", u"miljardinsilta"],
+	u"SallNm": [u"kymmenensille", u"sadansille", u"tuhannensille", u"miljoonansille", u"miljardinsille"],
+	u"SabNm": [u"kymmenensittä", u"sadansitta", u"tuhannensitta", u"miljoonansitta", u"miljardinsitta"],
+	u"SinNm": [u"kymmenensin", u"sadansin", u"tuhannensin", u"miljoonansin", u"miljardinsin"],
+	u"SkoNm": [u"kymmenensine", u"sadansine", u"tuhannensine", u"miljoonansine", u"miljardinsine"]
 }
 
 for sija in MULTI_ORDINALS.keys():
@@ -247,9 +257,13 @@ for sija in MULTI_ORDINALS.keys():
 	lexcFile.write(u"[Bc]" + diacritic + MULTI_ORDINALS[sija][1] + u":" + diacritic + MULTI_ORDINALS[sija][1] + u"\t" + lexiconName + u"_a\t;\n")
 	lexcFile.write(u"[Bc]" + diacritic + MULTI_ORDINALS[sija][2] + u":" + diacritic + MULTI_ORDINALS[sija][2] + u"\t" + lexiconName + u"_a\t;\n")
 	lexcFile.write(u"[Bc]" + diacritic + MULTI_ORDINALS[sija][3] + u":" + diacritic + MULTI_ORDINALS[sija][3] + u"\t" + lexiconName + u"_a\t;\n")
+	lexcFile.write(u"[Bc]" + diacritic + MULTI_ORDINALS[sija][4] + u":" + diacritic + MULTI_ORDINALS[sija][4] + u"\t" + lexiconName + u"_a\t;\n")
 	
 	numeralLines = []
-	numeralLines.append(diacritic + u":" + diacritic + u"\tJärjestyslukuLiitesana_<A>\t;")
+	if sija in [u"SgNy", u"SgNm"]:
+		numeralLines.append(diacritic + u":" + diacritic + u"\tJärjestyslukuLiitesanaJl_<A>\t;")
+	else:
+		numeralLines.append(diacritic + u":" + diacritic + u"\tJärjestyslukuLiitesana_<A>\t;")
 	numeralLines.append(diacritic + u":" + diacritic + u"\tJärjestyslukuToista\t;")
 	if OPTIONS["sukija"]:
 		numeralLines.append(diacritic + u":" + diacritic + u"\tSukijaJärjestyslukuKolmattaYhdeksättä\t;")
